/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.advis;

import dk.nsi.fmk.advis.AdvisClientFactory;
import dk.nsi.fmk.advis.AdvisService;
import dk.nsi.fmk.moduleframework.client.config.ClientApplicationConfig;
import dk.sds.ecpr2.util.Properties;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

public class AdvisModuleClientFactory {

    @Autowired
    Properties properties;

    private static volatile Boolean configured = null;
    private static final Logger log = LogManager.getLogger(AdvisModuleClientFactory.class);
    private static java.util.Properties consumerProperties = null;

    public AdvisModuleClientFactory(Properties properties) {
        this.properties = properties;
    }

    private synchronized void init() {
        if (configured == null) {
            try {
                String kafkaProducerProperties = properties.kafkaProducerProperties;
                log.info("Kafka AdvisClient loading properties from: " + properties.kafkaProducerProperties);
                consumerProperties = loadProperties(kafkaProducerProperties);
                configured = Boolean.TRUE;
                log.info("AdvisClient configured with kafka-producer.properties from classpath: " + consumerProperties.toString());

                if (configured == null) {
                    configured = Boolean.FALSE;
                    log.warn("Could not configure kafka AdvisClient - kafka-producer.properties not found: " + properties.kafkaProducerProperties);
                }
            } catch (Exception e) {
                configured = Boolean.FALSE;
                log.error("Could not configure kafka AdvisClient - got exception", e);
            }
        }
    }

    public AdvisService getAdvisServiceClient() {
        if (configured == null) {
            init();
        }
        return AdvisClientFactory.createAdvisClient(() -> properties.advisSendViaKafka, new ClientApplicationConfig("advis-client.properties"), consumerProperties);
    }

    private java.util.Properties loadProperties(String name) throws IOException {
        String fileName = System.getProperty("jboss.server.config.dir") + name;
        FileInputStream is = new FileInputStream(fileName);
        if (is == null) {
            throw new IOException("Could not load resource " + name + " from classpath");
        }

        java.util.Properties props = new java.util.Properties();
        props.load(is);
        return props;
    }
}
