/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.advis;

import dk.nsi.fmk.advis.AdvisService;
import dk.nsi.fmk.advis.proto.Advis;
import dk.nsi.fmk.moduleframework.data.ModuleFramework;
import dk.sds.ecpr2.domain.RequestContextVO;
import dk.sds.ecpr2.util.Properties;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.UUID;

@Log4j2
public class AsyncAdvisFacadeModuleImpl implements AsyncAdvisFacade {

	@Autowired
	Properties properties;

	private final String clientSystem = "eCPR";
	
	AdvisService advisService;

	public AsyncAdvisFacadeModuleImpl(AdvisService advisService) {
		this.advisService = advisService;
	}

	protected ModuleFramework.RequestContext getRequestContext(String messageId) {
		ModuleFramework.RequestContext.Builder builder = ModuleFramework.RequestContext.newBuilder();
		builder.setClientSystem(clientSystem);
		builder.setMessageId(messageId != null ? messageId : UUID.randomUUID().toString());
		return builder.build();
	}

	protected AdvisService getAdvisService() {
		return advisService;
	}

	@Override
	public void advisPersonIdUpdated(RequestContextVO context, String oid, String id) {
		if (!properties.advisEnabled) {
			log.warn("Advis forwarding is disabled - skipping");
			return;
		}
		Advis.PersonId personId = getPersonId(oid, id);
		getAdvisService().personIdUpdated(getRequestContext(context.getMessageId()), personId);
		log.debug("Sent updated advis for: " + oid + ":" + id);
	}

	@Override
	public void advisPersonIdDeleted(RequestContextVO context, String oid, String id) {
		if (!properties.advisEnabled) {
			log.warn("Advis forwarding is disabled - skipping");
			return;
		}
		Advis.PersonId personId = getPersonId(oid, id);
		getAdvisService().personIdDeleted(getRequestContext(context.getMessageId()), personId);
		log.debug("Sent updated advis for: " + oid + ":" + id);
	}

	private Advis.PersonId getPersonId(String oid, String id) {
		return Advis.PersonId.newBuilder().setPersonId(oid + ":" + id).build();
	}

}
