/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.BirthDateVO;
import dk.sds.ecpr2.domain.ContactVO;
import dk.sds.ecpr2.domain.GenderVO;

import java.time.Instant;
import java.util.Date;
import java.util.List;

public interface ContactDAO extends ECPR2DAO {
    List<ContactVO> getWithServiceActor(List<String> personUuids, List<Integer> personPids) throws DAOException;
    List<ContactVO> getWithServiceActor(String personUuid, Date date) throws DAOException;
    List<ContactVO> getAllContactsBetweenPersonPIDForSearch(int minPersonPid, int maxNoOfRows) throws DAOException;
    List<ContactVO> getAllContactsBetweenPersonPID(int minPersonPID, int maxPersonPID) throws DAOException;
    List<ContactVO> getAllContactsAfterModified(Instant modifiedTime) throws DAOException;
    List<ContactVO> getAllContactsAfterModifiedForSearch(Instant modifiedTime) throws DAOException;

    Integer createContact(ContactVO contact, Date nowTimestamp, int personId) throws DAOException;
    void createContacts(List<ContactVO> contacts) throws DAOException;
    void updateCurrentAndCreateNew(ContactVO vo, Date nowTimestamp, int personId) throws DAOException;
    void updateTimeStamp(ContactVO vo, Date nowTimestamp) throws DAOException;

}
