/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.*;

import java.time.Instant;
import java.util.Date;
import java.util.List;

/**
 * Data Access Object for Identifiers table
 */
public interface IdentifierDAO extends ECPR2DAO {
    List<IdentifierVO> getWithServiceActor(List<String> personUuids, List<Integer> personPids) throws DAOException;
    List<IdentifierVO> getWithServiceActor(String personUuid, Date date) throws DAOException;
    IdentifierWithoutServiceActorVO getCurrentIdentifierByValueAndOID(String value, String oid) throws DAOException;
    IdentifierWithoutServiceActorVO getIdentifierByValueAndOID(String value, String oid) throws DAOException;
    List<IdentifierWithoutServiceActorVO> getCurrentIdentifiersByValueAndOID(String value, String oid) throws DAOException;
    List<IdentifierWithoutServiceActorVO> getIdentifiersByValueAndOID(String value, String oid) throws DAOException;
    List<IdentifierVO> getAllIdentifiersBetweenPersonPIDForSearch(int minPersonPid, int maxNoOfRows) throws DAOException;
    List<IdentifierVO> getAllIdentifiersBetweenPersonPID(int minPersonPid, int maxNoOfRows) throws DAOException;
    List<IdentifierVO> getAllIdentifiersAfterModified(Instant modifiedTime) throws DAOException;
    List<IdentifierVO> getAllIdentifiersAfterModifiedForSearch(Instant modifiedTime) throws DAOException;
    /**
     * @param pattern is a MySQL LIKE query pattern.
     */
    List<IdentifierWithoutServiceActorVO> getMatchingIdentifiersWithoutServiceActor(String pattern, String oid) throws DAOException;

    void createIdentifiers(List<IdentifierVO> identifiers) throws DAOException;
    Integer createIdentifier(IdentifierVO identifier, Date now, int personId) throws DAOException;

    IdentifierWithoutServiceActorVO getCurrentIdentifierByKeyIdentifier(KeyIdentifierVO keyIdentifier) throws DAOException;

    void updateCurrentAndCreateNew(IdentifierVO identifier, Date now, int personId) throws DAOException;

    void updateIdentifier(IdentifierVO vo, Date now) throws DAOException;

    void updateTimeStamp(IdentifierVO vo, Date now) throws DAOException;

    List<IdentifierWithoutServiceActorVO> getIdentifiersByUuids(List<String> uuids, Date now) throws DAOException;
}
