/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.BirthDateVO;
import dk.sds.ecpr2.domain.ContactVO;
import dk.sds.ecpr2.domain.IdentifierVO;
import dk.sds.ecpr2.domain.NameVO;

import java.time.Instant;
import java.util.Date;
import java.util.List;

public interface NameDAO extends ECPR2DAO {
    List<NameVO> getWithServiceActor(List<String> personUuids, List<Integer> personPids) throws DAOException;
    List<NameVO> getWithServiceActor(String personUuid, Date date) throws DAOException;
    List<NameVO> getAllNamesBetweenPersonPIDForSearch(int minPersonPid, int maxNoOfRows) throws DAOException;
    List<NameVO> getAllNamesBetweenPersonPID(int minPersonPID, int maxPersonPID) throws DAOException;
    List<NameVO> getAllNamesAfterModified(Instant modifiedTime) throws DAOException;
    List<NameVO> getAllNamesAfterModifiedForSearch(Instant modifiedTime) throws DAOException;

    Integer createName(NameVO Name, Date now, int personId) throws DAOException;
    void createNames(List<NameVO> names) throws DAOException;
    void updateCurrentAndCreateNew(NameVO vo, Date now, int personId) throws DAOException;
    void updateTimeStamp(NameVO vo, Date now) throws DAOException;

}
