/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.PersonIdVO;
import dk.sds.ecpr2.domain.PersonVO;

import java.time.Instant;
import java.util.List;

public interface PersonDAO {
    List<PersonVO> getPersonsByUUIDs(List<String> personUuid) throws DAOException;
    PersonVO getCurrentPerson(String personUuid) throws DAOException;
    void createPersons(List<PersonVO> persons) throws DAOException;
    Integer createPerson(PersonVO p) throws DAOException;
    List<PersonVO> getPersonsModifiedAfter(Instant modifiedDateTimeUtc, boolean reservedUntil) throws DAOException;
    List<PersonVO> getPersonsAfterPID(int minPersonPID, int batchSize, boolean reservedUntil) throws DAOException;
    PersonVO getPersonByPID(Integer personPID) throws DAOException;
    List<PersonVO> getPersonByPIDs(List<Integer> personPID) throws DAOException;
    void deletePerson(String personuuid) throws DAOException;
    List<PersonVO> getPersonUuidByPersonIds(List<PersonIdVO> identifiers) throws DAOException;
    List<PersonVO> getPersonByReservedUntil(Instant date) throws DAOException;
    void deleteReservedIds(List<Integer> idList) throws DAOException;
    List<PersonVO> getReservedPersonByModifiedTime(Instant date) throws DAOException;
}
