/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao.impl;

import dk.sds.ecpr2.dao.BlacklistingDAO;
import dk.sds.ecpr2.domain.BlacklistedDeCPRVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.util.List;

@Repository
public class BlacklistingDAOImpl implements BlacklistingDAO {
    @Autowired private DataSource dataSource;
    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    @PostConstruct
    private void init() {
        namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    @Override
    public BlacklistedDeCPRVO isBlacklistedDeCPR(String oid, String deCPR) {
        final String sql = "SELECT oid FROM Blacklist WHERE deCPR = :deCPR";
        final MapSqlParameterSource parms = new MapSqlParameterSource();
        parms.addValue("deCPR", deCPR);
        List<String> oids = namedParameterJdbcTemplate.queryForList(sql, parms, String.class);
        return new BlacklistedDeCPRVO(deCPR, oids);
    }

    @Override
    public void blacklistDeCPR(String oid, String deCPR) {
        final String updateSql = "INSERT INTO Blacklist (deCPR, oid) VALUES (:deCPR, :oid)";
        final MapSqlParameterSource updateParms = new MapSqlParameterSource();
        updateParms.addValue("deCPR", deCPR);
        updateParms.addValue("oid", oid);
        namedParameterJdbcTemplate.update(updateSql, updateParms);
    }
}
