/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao.impl;

import dk.sds.ecpr2.dao.ExportHistoryDAO;
import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.abstr.AbstractECPRDAOImpl;
import dk.sds.ecpr2.domain.ExportHistoryVO;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.sql.Date;
import java.time.Instant;
import java.util.List;

@Repository
public class ExportHistoryDAOImpl extends AbstractECPRDAOImpl implements ExportHistoryDAO {
    public static final String TABLE_NAME = "ExportHistory";
    private final String INSERT_SQL = "INSERT INTO `ecpr2`.`ExportHistory` (`ExportNumber`, `ExportTime`) " +
            "VALUES (:ExportNumber, :ExportTime)";


    @Override
    public void dataChangedToExport(int exportNumber, Instant date) throws DAOException {
        try {
            ExportHistoryVO exportHistory = new ExportHistoryVO();
            exportHistory.setExportNumber(exportNumber);
            exportHistory.setExportTime(Date.from(date));
            final Integer pid = createSingleObject(INSERT_SQL, exportHistory);
            exportHistory.setExportId(pid);
        } catch (DAOException e) {
            throw new DAOException("An error occurred inserting export history record: ", e);
        }
    }

    @Override
    public ExportHistoryVO getLatestExport() throws DAOException {
        final String sql = "SELECT * FROM ExportHistory e ORDER BY e.exportNumber DESC LIMIT 1";
        List<ExportHistoryVO> list = queryForList(sql);
        return list.isEmpty() ? null : list.get(0);
    }

    @Override
    public String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected RowMapper getDefaultRowMapper() {
        return new ExportHistoryRowMapper();
    }
}
