/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;

import java.util.*;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.AddressDAOImpl;
import dk.sds.ecpr2.dao.impl.abstr.AbstractDAOImpl;
import dk.sds.ecpr2.domain.AddressVO;
import dk.sds.ecpr2.util.ValueSets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Component
@Order(value=10)
public class AddressGenerator extends PersonDescriptorAbstractGenerator<AddressVO> {

    @Autowired
    private AddressDAOImpl dao;

    @Value("${datagenerator.rowcount.address:100}")
    private int numberOfRows;

    @Override
    protected int getNumberOfRowsToGenerate() {
        return numberOfRows;
    }

    @Override
    protected AbstractDAOImpl getDao() {
        return dao;
    }

    @Override
    protected IdRange generate(Map<String, IdRange> previouslyGeneratedIds) throws DAOException {
        return generateForPersonDescriptorTable(previouslyGeneratedIds.get("Person"));
    }

    @Override
    protected AddressVO produceObject(int personId) {
        AddressVO addressVO = new AddressVO();
        addressVO.setType("Type");
        addressVO.setUse(ValueSets.addressUseValues[this.getRandomIntBetween(0, ValueSets.addressUseValues.length - 1)]);
        addressVO.setStreetAddressLine1("AddressLine1");
        addressVO.setStreetAddressLine2("AddressLine2");
        addressVO.setStreetAddressLine3("AddressLine3");
        addressVO.setStreetAddressLine4("AddressLine4");
        addressVO.setStreetAddressLine5("AddressLine5");
        addressVO.setStreetAddressLine6("AddressLine6");
        addressVO.setStreetAddressLine7("AddressLine7");
        addressVO.setStreetAddressLine8("AddressLine8");
        addressVO.setStreetAddressLine9("AddressLine9");
        addressVO.setStreetAddressLine10("AddressLine10");
        addressVO.setCity("City");
        addressVO.setDistrict("District");
        addressVO.setState("State");
        addressVO.setPostalCode("PostalCode");
        addressVO.setCountry("Country");
        addressVO.setPostBox("1234");

        addressVO.setUuid(UUID.randomUUID().toString());
        addressVO.setPersonPID(personId);
        addressVO.setExpiry(getRandomDateBetween(2018, 2019));
        addressVO.setValidFrom(getRandomDateBetween(2017, 2018));
        addressVO.setValidTo(getRandomDateBetween(2018, 2022));
        return addressVO;
    }

    @Override
    protected int createSingleObjectInDb(AddressVO singleObject) throws DAOException {
        return dao.createAddress(singleObject, singleObject.getValidFrom(), singleObject.getPersonPID());
    }

    @Override
    protected void batchCreateObjectsInDb(List<AddressVO> objects) throws DAOException {
        dao.createAddresses(objects);
    }
}
