/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.GenderDAOImpl;
import dk.sds.ecpr2.dao.impl.abstr.AbstractDAOImpl;
import dk.sds.ecpr2.domain.GenderVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.UUID;

@Component
@Order(value = 10)
public class GenderGenerator extends PersonDescriptorAbstractGenerator<GenderVO> {

    @Autowired
    private GenderDAOImpl dao;

    @Override
    protected AbstractDAOImpl getDao() {
        return dao;
    }

    @Value("${datagenerator.rowcount.gender:100}")
    private int numberOfRows;

    @Override
    protected int getNumberOfRowsToGenerate() {
        return numberOfRows;
    }

    @Override
    protected IdRange generate(Map<String, IdRange> previouslyGeneratedIds) throws DAOException {
        return generateForPersonDescriptorTable(previouslyGeneratedIds.get("Person"));
    }

    @Override
    protected GenderVO produceObject(int personId) {
        GenderVO genderVO = new GenderVO();
        genderVO.setValue(getRandomIntBetween(0, 1) == 1 ? "male" : "female");

        genderVO.setUuid(UUID.randomUUID().toString());
        genderVO.setPersonPID(personId);
        genderVO.setValidFrom(getRandomDateBetween(2017, 2018));
        genderVO.setValidTo(getRandomDateBetween(2018, 2022));
        return genderVO;
    }

    @Override
    protected int createSingleObjectInDb(GenderVO singleObject) throws DAOException {
        return dao.createGender(singleObject, singleObject.getValidFrom(), singleObject.getPersonPID());
    }

    @Override
    protected void batchCreateObjectsInDb(List<GenderVO> objects) throws DAOException {
        dao.createGenders(objects);
    }
}
