/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;
import dk.sds.ecpr2.domain.ServiceActorVO;
import java.util.Date;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.NameDAOImpl;
import dk.sds.ecpr2.dao.impl.abstr.AbstractDAOImpl;
import dk.sds.ecpr2.domain.NameVO;
import dk.sds.ecpr2.util.ValueSets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.UUID;

@Component
@Order(value=10)
public class NameGenerator extends PersonDescriptorAbstractGenerator<NameVO> {

    @Autowired
    private NameDAOImpl dao;

    @Override
    protected AbstractDAOImpl getDao() {
        return dao;
    }

    @Value("${datagenerator.rowcount.name:100}")
    private int numberOfRows;

    @Override
    protected int getNumberOfRowsToGenerate() {
        return numberOfRows;
    }

    @Override
    protected IdRange generate(Map<String, IdRange> previouslyGeneratedIds) throws DAOException {
        return generateForPersonDescriptorTable(previouslyGeneratedIds.get("Person"));
    }

    @Override
    protected NameVO produceObject(int personId) {
        NameVO nameVO = new NameVO();
        nameVO.setUse("official");
        nameVO.setFamilyName("FamilyName" + getRandomIntBetween(1000,9999));
        nameVO.setGivenName("GivenName" + getRandomIntBetween(1000,9999));

        nameVO.setUuid(UUID.randomUUID().toString());
        nameVO.setPersonPID(personId);
        nameVO.setExpiry(getRandomDateBetween(2018, 2019));
        nameVO.setValidFrom(getRandomDateBetween(2017, 2018));
        nameVO.setValidTo(getRandomDateBetween(2018, 2022));
        return nameVO;
    }

    @Override
    protected int createSingleObjectInDb(NameVO singleObject) throws DAOException {
        return dao.createName(singleObject, singleObject.getValidFrom(), singleObject.getPersonPID());
    }

    @Override
    protected void batchCreateObjectsInDb(List<NameVO> objects) throws DAOException {
        dao.createNames(objects);
    }
}
