/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.abstr.PersonDescriptorVO;

import java.util.ArrayList;
import java.util.List;

public abstract class PersonDescriptorAbstractGenerator<V extends PersonDescriptorVO> extends AbstractGenerator {

    protected abstract V produceObject(int personId);

    protected abstract int createSingleObjectInDb(V singleObject) throws DAOException;

    protected abstract void batchCreateObjectsInDb(List<V> objects) throws DAOException;


    protected IdRange generateForPersonDescriptorTable(IdRange personIdRange) throws DAOException {
        final List<V> objects = new ArrayList<>();
        final int numberOfPeopleGenerated = personIdRange.max - personIdRange.min;
        final double averageValuesPerPerson = (double) getNumberOfRowsToGenerate() / numberOfPeopleGenerated;

        for (int personId = personIdRange.min; personId < personIdRange.max; personId++) {
            double randomRowCount = getRandomRowCount(averageValuesPerPerson, 0.5);
            for (int i = 0; i < randomRowCount; i++) {
                objects.add(produceObject(personId));
            }
        }

        int maxIdBefore = createSingleObjectInDb(produceObject(getRandomIntBetween(personIdRange)));
        batchCreateObjectsInDb((objects));
        int maxIdAfter = createSingleObjectInDb(produceObject(getRandomIntBetween(personIdRange)));

        return new IdRange(maxIdBefore, maxIdAfter);
    }



}
