/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.PersonDAOImpl;
import dk.sds.ecpr2.dao.impl.abstr.AbstractDAOImpl;
import dk.sds.ecpr2.domain.PersonVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Component
@Order(value=2)
public class PersonGenerator extends AbstractGenerator {

    public static final int MAX_VERSIONS_PER_PERSON = 4;
    @Autowired
    private PersonDAOImpl dao;

    @Override
    protected AbstractDAOImpl getDao() {
        return dao;
    }

    @Value("${datagenerator.rowcount.person:100}")
    private int numberOfRows;

    @Override
    protected int getNumberOfRowsToGenerate() {
        return numberOfRows;
    }

    @Override
    protected int getMinimumIdToRemove() {
        return 3;  // Rows are created manually in SQL scripts
    }

    @Override
    protected IdRange generate(Map<String, IdRange> previouslyGeneratedIds) throws DAOException {
        IdRange serviceActorIdRange = previouslyGeneratedIds.get("ServiceActor");

        List<PersonVO> persons = new ArrayList<>();
        int numberOfVersions;
        for (int n = 0; n < getNumberOfRowsToGenerate(); n += numberOfVersions) {
            numberOfVersions = getRandomIntBetween(1, MAX_VERSIONS_PER_PERSON);
            String uuid = UUID.randomUUID().toString();
            for (int version = 1; version <= numberOfVersions; version++) {
                persons.add(buildTestPerson(serviceActorIdRange, uuid, version));
            }
        }

        int maxIdBefore = dao.createPerson(buildTestPerson(serviceActorIdRange, UUID.randomUUID().toString(), 1));
        dao.createPersons(persons);
        int maxIdAfter = dao.createPerson(buildTestPerson(serviceActorIdRange, UUID.randomUUID().toString(), 1));

        return new IdRange(maxIdBefore, maxIdAfter);
    }


    private PersonVO buildTestPerson(IdRange serviceActorIdRange, String uuid, int version) {
        PersonVO personVO = new PersonVO();
        personVO.setPersonUUID(uuid);
        personVO.setServiceActorPID(getRandomIntBetween(serviceActorIdRange));
        personVO.setVersion(version);
        personVO.setVersionDescription("ÆØÅæøå");
        personVO.setReservedUntil(getRandomDateBetween(2018, 2025));
        return personVO;
    }

}
