/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.datagenerator;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.dao.impl.ServiceActorDAOImpl;
import dk.sds.ecpr2.dao.impl.abstr.AbstractDAOImpl;
import dk.sds.ecpr2.domain.ServiceActorVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
@Order(value=1)
public class ServiceActorGenerator extends AbstractGenerator {

    @Autowired
    private ServiceActorDAOImpl dao;

    @Override
    protected AbstractDAOImpl getDao() {
        return dao;
    }

    @Value("${datagenerator.rowcount.serviceactor:100}")
    private int numberOfRows;

    @Override
    protected int getNumberOfRowsToGenerate() {
        return numberOfRows;
    }

    @Override
    protected int getMinimumIdToRemove() {
        return 2; // Rows are created manually in SQL scripts
    }

    @Override
    protected IdRange generate(Map<String, IdRange> previouslyGeneratedIds) throws DAOException {
        List<ServiceActorVO> serviceActors = new ArrayList<>();

        for (int i = 0; i < getNumberOfRowsToGenerate(); i++) {
            serviceActors.add(buildTestServiceActorVO());
        }

        Date now = new Date();

        int maxIdBefore = dao.createServiceActor(buildTestServiceActorVO(), now);
        dao.createServiceActors(serviceActors);
        int maxIdAfter = dao.createServiceActor(buildTestServiceActorVO(), now);

        return new IdRange(maxIdBefore, maxIdAfter);
    }

    private ServiceActorVO buildTestServiceActorVO() {
        final ServiceActorVO serviceActorVO = new ServiceActorVO();
        serviceActorVO.setPersonIdentifier(getRandomCPRNumber());
        serviceActorVO.setPersonIdentifierOID("1.2.208.176.1.2");
        serviceActorVO.setFirstName("Firstname" + getRandomIntBetween(100,999));
        serviceActorVO.setLastName("Lastname" + getRandomIntBetween(100,999));
        serviceActorVO.setRole("Rolle");
        serviceActorVO.setAuthorisationNumber("A" + getRandomIntBetween(10, 99));
        serviceActorVO.setOrganizationId("OrgId" + getRandomIntBetween(100000, 999999));
        serviceActorVO.setOrganizationName("Region" + getRandomIntBetween(1000,9999));
        serviceActorVO.setModifiedTime(new Date());

        return serviceActorVO;
    }

    private String getRandomCPRNumber() {
        final String randomNumber = String.format("%02d%02d%02d%04d",
                getRandomIntBetween(1, 31),
                getRandomIntBetween(1, 12),
                getRandomIntBetween(0, 99),
                getRandomIntBetween(1000, 9999));
        return randomNumber;
    }

}
