/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import dk.sds.ecpr2.constants.SearchConstants;
import dk.sds.ecpr2.domain.abstr.PersonDescriptorWithServiceActorVO;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Data
@EqualsAndHashCode(callSuper = false, exclude = {"addressPID", "personUUID"})
@JsonInclude(JsonInclude.Include.NON_NULL)
public class AddressVO extends PersonDescriptorWithServiceActorVO {
    @JsonIgnore
    private Integer addressPID;
    private String type;
    private String use;
    private String streetAddressLine1;
    private String streetAddressLine2;
    private String streetAddressLine3;
    private String streetAddressLine4;
    private String streetAddressLine5;
    private String streetAddressLine6;
    private String streetAddressLine7;
    private String streetAddressLine8;
    private String streetAddressLine9;
    private String streetAddressLine10;
    private String city;
    private String district;
    private String state;
    private String postalCode;
    private String country;
    private String postBox;
    private Date expiry;
    private String personUUID;

    @JsonIgnore
    public List<LuceneKeyValue> getLuceneList() {
        List<LuceneKeyValue> luceneKeyValueList = new ArrayList<>();

        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine1));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine2));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine3));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine4));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine5));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine6));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine7));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine8));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine9));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STREET_ADDRESS, streetAddressLine10));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.CITY, city));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.DISTRICT, district));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.STATE, state));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.POSTAL_CODE, postalCode));
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.COUNTRY, country));
        return luceneKeyValueList;
    }
}
