/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import dk.sds.ecpr2.constants.SearchConstants;
import dk.sds.ecpr2.domain.abstr.PersonDescriptorWithServiceActorVO;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.ArrayList;
import java.util.List;

@Data
@EqualsAndHashCode(callSuper = false, exclude = {"birthDatePID", "personUUID"})
@JsonInclude(JsonInclude.Include.NON_NULL)
public class BirthDateVO extends PersonDescriptorWithServiceActorVO {

    @JsonIgnore
    private Integer birthDatePID;
    private Integer day;
    private Integer month;
    private Integer year;
    private String personUUID;

    @JsonIgnore
    public List<LuceneKeyValue> getLuceneList() {
        List<LuceneKeyValue> luceneKeyValueList = new ArrayList<>();
        String birthDate = mapBirthDateValue(year, month, day);
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.BIRTH_DATE, birthDate));
        return luceneKeyValueList;
    }

    private String mapBirthDateValue(Integer year, Integer month, Integer day) {
        return year +
                (month != null && month != 0 ? "-" + (month < 10 ? "0" + month : month) : "") +
                (day != null && day != 0 ? "-" + (day < 10 ? "0" + day : day) : "");
    }

}
