/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.NoSuchElementException;
import java.util.function.Predicate;

@AllArgsConstructor
@Getter
public enum CountryCode {
    AFG("Afghanistan", 4, "AFG", "AF"),
    ALB("Albanien", 8, "ALB", "AL"),
    DZA("Algeriet", 12, "DZA", "DZ"),
    ASM("Amerikansk Samoa", 16, "ASM", "AS"),
    USA("USA", 840, "USA", "US"),
    AND("Andorra", 20, "AND", "AD"),
    AGO("Angola", 24, "AGO", "AO"),
    AIA("Anguilla", 660, "AIA", "AI"),
    ATA("Antarktis Antarktika", 10, "ATA", "AQ"),
    ATG("Antigua og Barbuda", 28, "ATG", "AG"),
    ARG("Argentina", 32, "ARG", "AR"),
    ARM("Armenien", 51, "ARM", "AM"),
    ABW("Aruba", 533, "ABW", "AW"),
    AUS("Australien", 36, "AUS", "AU"),
    AZE("Aserbajdsjan", 31, "AZE", "AZ"),
    BHS("Bahamas", 44, "BHS", "BS"),
    BHR("Bahrain", 48, "BHR", "BH"),
    BGD("Bangladesh", 50, "BGD", "BD"),
    BRB("Barbados", 52, "BRB", "BB"),
    BEL("Belgien", 56, "BEL", "BE"),
    BLZ("Belize", 84, "BLZ", "BZ"),
    BEN("Benin", 204, "BEN", "BJ"),
    BMU("Bermuda", 60, "BMU", "BM"),
    BTN("Bhutan", 64, "BTN", "BT"),
    BOL("Bolivia", 68, "BOL", "BO"),
    BIH("Bosnien-Hercegovina", 70, "BIH", "BA"),
    BWA("Botswana", 72, "BWA", "BW"),
    BVT("Bouvetøen", 74, "BVT", "BV"),
    BRA("Brasilien", 76, "BRA", "BR"),
    IOT("British Indian Ocean Territory", 86, "IOT", "IO"),
    BRN("Brunei", 96, "BRN", "BN"),
    BGR("Bulgarien", 100, "BGR", "BG"),
    BFA("Burkina Faso", 854, "BFA", "BF"),
    BDI("Burundi", 108, "BDI", "BI"),
    CAN("Canada", 124, "CAN", "CA"),
    CYM("Caymanøerne", 136, "CYM", "KY"),
    CHL("Chile", 152, "CHL", "CL"),
    CXR("Juleøen", 162, "CXR", "CX"),
    COL("Colombia", 170, "COL", "CO"),
    COK("Cookøerne", 184, "COK", "CK"),
    CRI("Costa Rica", 188, "CRI", "CR"),
    CUB("Cuba", 192, "CUB", "CU"),
    CUW("Curaçao", 531, "CUW", "CW"),
    DNK("Danmark", 208, "DNK", "DK"),
    DJI("Djibouti", 262, "DJI", "DJ"),
    DMA("Dominica", 212, "DMA", "DM"),
    DOM("Dominikanske Republik", 214, "DOM", "DO"),
    ECU("Ecuador", 218, "ECU", "EC"),
    EGY("Egypten", 818, "EGY", "EG"),
    CIV("Elfenbenskysten", 384, "CIV", "CI"),
    GNQ("Ækvatorialguinea", 226, "GNQ", "GQ"),
    SLV("El Salvador", 222, "SLV", "SV"),
    ERI("Eritrea", 232, "ERI", "ER"),
    EST("Estland", 233, "EST", "EE"),
    ETH("Etiopien", 231, "ETH", "ET"),
    FLK("Falklandsøerne", 238, "FLK", "FK"),
    FJI("Fiji", 242, "FJI", "FJ"),
    PHL("Filippinerne", 608, "PHL", "PH"),
    FIN("Finland", 246, "FIN", "FI"),
    FRA("Frankrig", 250, "FRA", "FR"),
    GUF("Fransk Guiana", 254, "GUF", "GF"),
    PYF("Fransk Polynesien", 258, "PYF", "PF"),
    ATF("Franske Sydterritorier", 260, "ATF", "TF"),
    FRO("Færøerne", 234, "FRO", "FO"),
    GAB("Gabon", 266, "GAB", "GA"),
    GMB("Gambia", 270, "GMB", "GM"),
    GEO("Georgien", 268, "GEO", "GE"),
    GHA("Ghana", 288, "GHA", "GH"),
    GIB("Gibraltar", 292, "GIB", "GI"),
    GRD("Grenada", 308, "GRD", "GD"),
    GRL("Grønland", 304, "GRL", "GL"),
    GLP("Guadeloupe", 312, "GLP", "GP"),
    GUM("Guam", 316, "GUM", "GU"),
    GTM("Guatemala", 320, "GTM", "GT"),
    GIN("Guinea", 324, "GIN", "GN"),
    GNB("Guinea-Bissau", 624, "GNB", "GW"),
    GUY("Guyana", 328, "GUY", "GY"),
    HTI("Haiti", 332, "HTI", "HT"),
    HMD("Heard-øen og McDonald-øerne", 334, "HMD", "HM"),
    GRC("Grækenland", 300, "GRC", "GR"),
    HND("Honduras", 340, "HND", "HN"),
    HKG("Hongkong", 344, "HKG", "HK"),
    BLR("Hviderusland", 112, "BLR", "BY"),
    IND("Indien", 356, "IND", "IN"),
    IDN("Indonesien", 360, "IDN", "ID"),
    IRQ("Irak", 368, "IRQ", "IQ"),
    IRN("Iran", 364, "IRN", "IR"),
    IRL("Irland", 372, "IRL", "IE"),
    ISL("Island", 352, "ISL", "IS"),
    ISR("Israel", 376, "ISR", "IL"),
    ITA("Italien", 380, "ITA", "IT"),
    JAM("Jamaica", 388, "JAM", "JM"),
    JPN("Japan", 392, "JPN", "JP"),
    YEM("Yemen", 887, "YEM", "YE"),
    VGB("Britiske Jomfruøer", 92, "VGB", "VG"),
    VIR("Amerikanske Jomfruøer", 850, "VIR", "VI"),
    JOR("Jordan", 400, "JOR", "JO"),
    KHM("Cambodja", 116, "KHM", "KH"),
    CMR("Cameroun", 120, "CMR", "CM"),
    CPV("Kap Verde", 132, "CPV", "CV"),
    BES("Nederlandske Antiller", 535, "BES", "BQ"),
    KAZ("Kasakhstan", 398, "KAZ", "KZ"),
    KEN("Kenya", 404, "KEN", "KE"),
    CHN("Kina", 156, "CHN", "CN"),
    KGZ("Kirgisistan", 417, "KGZ", "KG"),
    KIR("Kiribati", 296, "KIR", "KI"),
    CCK("Cocosøerne", 166, "CCK", "CC"),
    COM("Comorerne", 174, "COM", "KM"),
    COG("Congo", 178, "COG", "CG"),
    COD("Demokratiske Republik Congo", 180, "COD", "CD"),
    HRV("Kroatien", 191, "HRV", "HR"),
    KWT("Kuwait", 414, "KWT", "KW"),
    CYP("Cypern", 196, "CYP", "CY"),
    LAO("Laos", 418, "LAO", "LA"),
    LVA("Letland", 428, "LVA", "LV"),
    LSO("Lesotho", 426, "LSO", "LS"),
    LBN("Libanon", 422, "LBN", "LB"),
    LBR("Liberia", 430, "LBR", "LR"),
    LBY("Libyen", 434, "LBY", "LY"),
    LIE("Liechtenstein", 438, "LIE", "LI"),
    LTU("Litauen", 440, "LTU", "LT"),
    LUX("Luxembourg", 442, "LUX", "LU"),
    MAC("Macao", 446, "MAC", "MO"),
    MDG("Madagaskar", 450, "MDG", "MG"),
    MKD("Makedonien", 807, "MKD", "MK"),
    MWI("Malawi", 454, "MWI", "MW"),
    MYS("Malaysia", 458, "MYS", "MY"),
    MDV("Maldiverne", 462, "MDV", "MV"),
    MLI("Mali", 466, "MLI", "ML"),
    MLT("Malta", 470, "MLT", "MT"),
    MAR("Marokko", 504, "MAR", "MA"),
    MHL("Marshalløerne", 584, "MHL", "MH"),
    MTQ("Martinique", 474, "MTQ", "MQ"),
    MRT("Mauretanien", 478, "MRT", "MR"),
    MUS("Mauritius", 480, "MUS", "MU"),
    MYT("Mayotte", 175, "MYT", "YT"),
    MEX("Mexico", 484, "MEX", "MX"),
    FSM("Mikronesien", 583, "FSM", "FM"),
    MDA("Moldova", 498, "MDA", "MD"),
    MCO("Monaco", 492, "MCO", "MC"),
    MNG("Mongoliet", 496, "MNG", "MN"),
    MNE("Montenegro", 499, "MNE", "ME"),
    MSR("Montserrat", 500, "MSR", "MS"),
    MOZ("Mozambique", 508, "MOZ", "MZ"),
    MMR("Burma", 104, "MMR", "MM"),
    NAM("Namibia", 516, "NAM", "NA"),
    NRU("Nauru", 520, "NRU", "NR"),
    NLD("Holland", 528, "NLD", "NL"),
    NPL("Nepal", 524, "NPL", "NP"),
    NZL("New Zealand", 554, "NZL", "NZ"),
    NIC("Nicaragua", 558, "NIC", "NI"),
    NER("Niger", 562, "NER", "NE"),
    NGA("Nigeria", 566, "NGA", "NG"),
    NIU("Niue", 570, "NIU", "NU"),
    PRK("Nordkorea", 408, "PRK", "KP"),
    MNP("Nordmarianerne", 580, "MNP", "MP"),
    NFK("Norfolk Island", 574, "NFK", "NF"),
    NOR("Norge", 578, "NOR", "NO"),
    NCL("Ny Kaledonien", 540, "NCL", "NC"),
    OMN("Oman", 512, "OMN", "OM"),
    PAK("Pakistan", 586, "PAK", "PK"),
    PLW("Palau", 585, "PLW", "PW"),
    PSE("Palæstina", 275, "PSE", "PS"),
    PAN("Panama", 591, "PAN", "PA"),
    PNG("Papua Ny Guinea", 598, "PNG", "PG"),
    PRY("Paraguay", 600, "PRY", "PY"),
    PER("Peru", 604, "PER", "PE"),
    PCN("Pitcairn", 612, "PCN", "PN"),
    POL("Polen", 616, "POL", "PL"),
    PRT("Portugal", 620, "PRT", "PT"),
    PRI("Puerto Rico", 630, "PRI", "PR"),
    QAT("Qatar", 634, "QAT", "QA"),
    REU("Réunion", 638, "REU", "RE"),
    ROU("Rumænien", 642, "ROU", "RO"),
    RUS("Rusland", 643, "RUS", "RU"),
    RWA("Rwanda", 646, "RWA", "RW"),
    SHN("Sankt Helena", 654, "SHN", "SH"),
    KNA("Saint Kitts og Nevis", 659, "KNA", "KN"),
    LCA("Saint Lucia", 662, "LCA", "LC"),
    SPM("Saint-Pierre og Miquelon", 666, "SPM", "PM"),
    VCT("Saint Vincent og Grenadinerne", 670, "VCT", "VC"),
    SLB("Salomonøerne", 90, "SLB", "SB"),
    WSM("Samoa", 882, "WSM", "WS"),
    SMR("San Marino", 674, "SMR", "SM"),
    STP("São Tomé og Príncipe", 678, "STP", "ST"),
    SAU("Saudi-Arabien", 682, "SAU", "SA"),
    SEN("Senegal", 686, "SEN", "SN"),
    CAF("Centralafrikanske Republik", 140, "CAF", "CF"),
    SRB("Serbien", 688, "SRB", "RS"),
    SYC("Seychellerne", 690, "SYC", "SC"),
    SLE("Sierra Leone", 694, "SLE", "SL"),
    SGP("Singapore", 702, "SGP", "SG"),
    SXM("Sint Maarten", 534, "SXM", "SX"),
    SVK("Slovakiet", 703, "SVK", "SK"),
    SVN("Slovenien", 705, "SVN", "SI"),
    SOM("Somalia", 706, "SOM", "SO"),
    ESP("Spanien", 724, "ESP", "ES"),
    LKA("Sri Lanka", 144, "LKA", "LK"),
    GBR("Storbritannien", 826, "GBR", "GB"),
    SDN("Sudan", 736, "SDN", "SD"),
    SUR("Surinam", 740, "SUR", "SR"),
    SJM("Norge Svalbard og Jan Mayen", 744, "SJM", "SJ"),
    SWE("Sverige", 752, "SWE", "SE"),
    CHE("Schweiz", 756, "CHE", "CH"),
    SWZ("Swaziland", 748, "SWZ", "SZ"),
    SYR("Syrien", 760, "SYR", "SY"),
    ZAF("Sydafrika", 710, "ZAF", "ZA"),
    SGS("South Georgia og South Sandwich Islands", 239, "SGS", "GS"),
    KOR("Sydkorea", 410, "KOR", "KR"),
    SSD("Sydsudan", 728, "SSD", "SS"),
    TWN("Republikken Kina Taiwan", 158, "TWN", "TW"),
    TJK("Tadsjikistan", 762, "TJK", "TJ"),
    TZA("Tanzania", 834, "TZA", "TZ"),
    THA("Thailand", 764, "THA", "TH"),
    TLS("Østtimor", 626, "TLS", "TL"),
    TGO("Togo", 768, "TGO", "TG"),
    TKL("Tokelau", 772, "TKL", "TK"),
    TON("Tonga", 776, "TON", "TO"),
    TTO("Trinidad og Tobago", 780, "TTO", "TT"),
    TCD("Tchad", 148, "TCD", "TD"),
    CZE("Tjekkiet", 203, "CZE", "CZ"),
    TUN("Tunesien", 788, "TUN", "TN"),
    TUR("Tyrkiet", 792, "TUR", "TR"),
    TKM("Turkmenistan", 795, "TKM", "TM"),
    TCA("Turks- og Caicosøerne", 796, "TCA", "TC"),
    DEU("Tyskland", 276, "DEU", "DE"),
    TUV("Tuvalu", 798, "TUV", "TV"),
    UGA("Uganda", 800, "UGA", "UG"),
    UKR("Ukraine", 804, "UKR", "UA"),
    HUN("Ungarn", 348, "HUN", "HU"),
    ARE("Forenede Arabiske Emirater", 784, "ARE", "AE"),
    UMI("USA's ydre småøer", 581, "UMI", "UM"),
    URY("Uruguay", 858, "URY", "UY"),
    UZB("Usbekistan", 860, "UZB", "UZ"),
    VUT("Vanuatu", 548, "VUT", "VU"),
    VAT("Vatikanstaten", 336, "VAT", "VA"),
    VEN("Venezuela", 862, "VEN", "VE"),
    ESH("Vestsahara", 732, "ESH", "EH"),
    VNM("Vietnam", 704, "VNM", "VN"),
    WLF("Wallis og Futuna", 876, "WLF", "WF"),
    ZMB("Zambia", 894, "ZMB", "ZM"),
    ZWE("Zimbabwe", 716, "ZWE", "ZW"),
    AUT("Østrig", 40, "AUT", "AT"),
    ALA("Ålandsøerne", 248, "ALA", "AX"),
    XKX("Kosovo", 983, "XKX", "XK");

    private final String fullName;
    private final int numericValue;
    private final String threeLetterAbbreviation;
    private final String twoLetterAbbreviation;

    public static CountryCode getByFullName(String fullName) {
        return getCountryCodeByPredicate(countryCode -> countryCode.fullName.equals(fullName));
    }

    public static CountryCode getByTwoLetterAbbreviation(String abbreviation) {
        return getCountryCodeByPredicate(countryCode -> countryCode.twoLetterAbbreviation.equals(abbreviation));
    }

    public static CountryCode getByThreeLetterAbbreviation(String abbreviation) {
        return getCountryCodeByPredicate(countryCode -> countryCode.threeLetterAbbreviation.equals(abbreviation));
    }

    public static CountryCode getByNumericValue(int value) {
        return getCountryCodeByPredicate(countryCode -> countryCode.numericValue == value);
    }

    public static Boolean isFullNameValid(String fullName) {
        return getByFullName(fullName) != null;
    }

    public static Boolean isTwoLetterAbbreviationValid(String abbreviation) {
        return getByTwoLetterAbbreviation(abbreviation) != null;
    }

    public static Boolean isThreeLetterAbbreviationValid(String abbreviation) {
        return getByThreeLetterAbbreviation(abbreviation) != null;
    }

    public static Boolean isNumericValueValid(String value) {
        try {
            return isNumericValueValid(Integer.parseInt(value));
        } catch (NumberFormatException e) {
            return false;
        }
    }

    public static Boolean isNumericValueValid(int value) {
        return getByNumericValue(value) != null;
    }

    private static CountryCode getCountryCodeByPredicate(Predicate<CountryCode> predicate) {
        try {
            return Arrays.stream(CountryCode.values()).filter(predicate).findFirst().get();
        } catch (NoSuchElementException e) {
            return null;
        }
    }

    public static String countryCodeToCountryName(String country) {
        if (CountryCode.isTwoLetterAbbreviationValid(country)) {
            return getByTwoLetterAbbreviation(country).getFullName();
        } else if (CountryCode.isThreeLetterAbbreviationValid(country)) {
            return getByThreeLetterAbbreviation(country).getFullName();
        } else if (CountryCode.isNumericValueValid(country)) {
            return getByNumericValue(Integer.parseInt(country)).getFullName();
        } else {
            return null;
        }
    }

}
