/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;


import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import dk.sds.ecpr2.Modified;
import dk.sds.ecpr2.constants.SearchConstants;
import dk.sds.ecpr2.domain.abstr.PersonDescriptorWithServiceActorVO;
import lombok.Data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

@Data
@JsonInclude(JsonInclude.Include.NON_EMPTY)
public class PersonVO extends PersonDescriptorWithServiceActorVO {

    private KeyIdentifierVO keyIdentifier;
    @JsonIgnore
    private Integer personPID;
    @JsonIgnore
    private Integer serviceActorPID;
    @JsonIgnore
    private Integer modifierPID;
    private String personUUID;
    private Integer version;
    private String versionDescription;
    private Date reservedUntil;
    private Date modifiedTime;
    private String status;

    private List<Modified> modifiers = new ArrayList<>();
    private List<IdentifierVO> identifiers = new ArrayList<>();
    private List<NameVO> names = new ArrayList<>();
    private List<GenderVO> genders = new ArrayList<>();
    private List<BirthDateVO> birthDates = new ArrayList<>();
    private List<ContactVO> contacts = new ArrayList<>();
    private List<AddressVO> addresses = new ArrayList<>();

    @JsonIgnore
    public List<LuceneKeyValue> getLuceneList() {
        List<LuceneKeyValue> luceneKeyValueList = new ArrayList<>();
        luceneKeyValueList.add(new LuceneKeyValue(SearchConstants.PERSON_UUID, personUUID));
        for (NameVO name : names) {
            luceneKeyValueList.addAll(name.getLuceneList());
        }
        for (GenderVO gender : genders) {
            luceneKeyValueList.addAll(gender.getLuceneList());
        }
        for (BirthDateVO birthDate : birthDates) {
            luceneKeyValueList.addAll(birthDate.getLuceneList());
        }
        for (ContactVO contact : contacts) {
            luceneKeyValueList.addAll(contact.getLuceneList());
        }
        for (AddressVO address : addresses) {
            luceneKeyValueList.addAll(address.getLuceneList());
        }

        return luceneKeyValueList;
    }

    public static int getMaxPersonPID(List<PersonVO> personBatch) {
        return personBatch
                .stream()
                .mapToInt(PersonVO::getPersonPID)
                .max()
                .orElse(0);
    }

    public static int getMinPersonPID(List<PersonVO> personBatch) {
        return personBatch
                .stream()
                .mapToInt(PersonVO::getPersonPID)
                .min()
                .orElse(Integer.MAX_VALUE);
    }

}
