/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.kafka;

import dk.sds.ecpr2.kafka.vo.PersonIdentifier;
import dk.sds.ecpr2.util.Properties;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;
import org.springframework.kafka.support.serializer.JsonSerializer;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Configuration
public class KafkaProducerConfig {
    @Autowired private Properties properties;

    @Bean
    public KafkaTemplate<String, List<PersonIdentifier>> personIdUpdatedKafkaTemplate() {
        return new KafkaTemplate<>(producerFactoryPersonIdUpdated());
    }

    @Bean
    public ProducerFactory<String, List<PersonIdentifier>> producerFactoryPersonIdUpdated() {
        return new DefaultKafkaProducerFactory<>(createConfigProperties(), new StringSerializer(), new JsonSerializer<>());
    }

    private Map<String, Object> createConfigProperties() {
        Map<String, Object> configProps = new HashMap<>();
        configProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, properties.kafkaFmkServer);
        configProps.put(ProducerConfig.REQUEST_TIMEOUT_MS_CONFIG,"15000");
        configProps.put(ProducerConfig.MAX_BLOCK_MS_CONFIG,"15000");
        configProps.put(ProducerConfig.COMPRESSION_TYPE_CONFIG,"gzip");
        configProps.put(ProducerConfig.ACKS_CONFIG, "0");
        configProps.put(ProducerConfig.RETRIES_CONFIG, "0");
        configProps.put(ProducerConfig.LINGER_MS_CONFIG, "0");
        configProps.put(ProducerConfig.BATCH_SIZE_CONFIG, "0");
        return configProps;

    }
}
