/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.security;

import dk.sds.ecpr2.dao.StamdataFacade;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

public abstract class AuthorisationNumberAuthenticator implements Authenticator {

    @Autowired private StamdataFacade stamdataFacade;

    @Override
    public boolean authenticate(SecurityCredentials sc) {
        boolean notDoctorNotDentist = !("7170".equals(getEducationCode()) || "5433".equals(getEducationCode()));
        boolean cvrIsNullOrHasValue = sc.getCvr() == null || sc.getCvr().trim().length() == 0;
        if ((notDoctorNotDentist && cvrIsNullOrHasValue)
                || (sc.getIdCardType() != null && !"user".equals(sc.getIdCardType()))) {
            return false;
        }

        List<AuthorisationVO> authorisationVOs = stamdataFacade.getAuthorisationsForCpr(sc.getUserCpr());

        if (sc.getAuthorisationId() != null) {
            for (AuthorisationVO authorisation : authorisationVOs) {
                if (sc.getAuthorisationId().equals(authorisation.getAuthorisationId())) {
                    return authorisation.getEducationCode().equals(getEducationCode());
                }
            }
        } else {
            for (AuthorisationVO authorisation : authorisationVOs) {
                return authorisation.getEducationCode().equals(getEducationCode());
            }
        }
        return false;
    }

    public abstract String getEducationCode();
}
