/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service;

import dk.sds.ecpr2.KeyIdentifier;
import dk.sds.ecpr2.Modified;
import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.*;
import dk.sds.ecpr2.security.Permission;

import java.util.List;
import java.util.Map;

public interface ECPR2Service {

    PersonVO createPerson(PersonVO personVO, ServiceActorVO actor, RequestContextVO context);

    List<PersonVO> searchPersons(String searchStr) throws Exception;

    List<List<PersonVO>> reservePersonIds(List<ReserveRequestVO> reserveRequest, ServiceActorVO actor, RequestContextVO context) throws DAOException;

    PersonVO updatePerson(PersonVO personUpdate, ServiceActorVO serviceActor, RequestContextVO context);

    List<PersonVO> getPersonsById(List<PersonVO> persons, ServiceActorVO actor, RequestContextVO context) throws DAOException;

    void deletePerson(List<IdentifierVO> identifiers, Modified modified, ServiceActorVO actor, RequestContextVO context) throws Exception;

    PersonVO createPersonId(PersonVO personVO, ServiceActorVO actor, RequestContextVO context) throws DAOException;

    PersonVO mergePerson(KeyIdentifierVO from, KeyIdentifierVO to, Modified modified, ServiceActorVO actor, RequestContextVO context);

    List<OIDVO> getOIDs() throws DAOException;

    Map<String, List<Permission>> getPermissions(boolean getAllPermissions, boolean getCallersPermissions);

    int countPersons(String searchQuery) throws Exception;

    BlacklistedDeCPRVO isBlacklistedDeCPR(String oid, String deCPR);

    void blacklistDeCPR(String oid, String deCPR);

    Integer compareName(KeyIdentifier keyIdentifier, String name, ServiceActorVO actor, RequestContextVO context);
}
