/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.admin.impl;

import dk.nsi.fmk.admin.Admin;
import dk.nsi.fmk.moduleframework.data.ModuleFramework;
import dk.sds.ecpr2.security.RequestContext;
import dk.sds.ecpr2.service.admin.AdministrationFacade;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
@Log4j2
public class AdminModuleAdministrationFacadeImpl implements AdministrationFacade {
    private static final String CLIENT_SYSTEM = "eCPR";

    @Autowired
    private Admin admin; // created using the Spring-configured Factory

    @Override
    public boolean isSupporterUser(String userCPR) {
        dk.nsi.fmk.admin.admin.data.proto.Admin.AdminUserVOResponse adminUserWithWildcardCVR = admin.findAdminUserWithWildcardCVR(getRequestContext(),
                toProtocString(userCPR), toMRole(Role.ECPRCoordinator), toProtocString(CLIENT_SYSTEM));
        return (adminUserWithWildcardCVR != null && adminUserWithWildcardCVR.hasAdminUserVO() &&
                adminUserWithWildcardCVR.getAdminUserVO().hasCpr() && userCPR.equalsIgnoreCase(adminUserWithWildcardCVR.getAdminUserVO().getCpr()));

    }

    @Override
    public boolean canFindAdminUser(String userCPR, String cvr, Role role, String systemName) {
        return admin.findAdminUser(getRequestContext(), toProtocString(userCPR), toProtocString(cvr), toMRole(role), toProtocString(systemName)) != null;
    }

    private ModuleFramework.RequestContext getRequestContext() {
        ModuleFramework.RequestContext.Builder builder = ModuleFramework.RequestContext.newBuilder();
        builder.setClientSystem(CLIENT_SYSTEM);
        if (RequestContext.get() != null && RequestContext.get().getMessageId() != null) {
            builder.setMessageId(RequestContext.get().getMessageId());
        }
        return builder.build();
    }

    private static dk.nsi.fmk.admin.admin.data.proto.Admin.ProtocString toProtocString(String s) {
        return dk.nsi.fmk.admin.admin.data.proto.Admin.ProtocString.newBuilder().setValue(s).build();
    }

    private static dk.nsi.fmk.admin.admin.data.proto.Admin.Role toMRole(Role role) {
        dk.nsi.fmk.admin.admin.data.proto.Admin.Role.Builder builder = dk.nsi.fmk.admin.admin.data.proto.Admin.Role.newBuilder();
        builder.setRole((int) role.getID());
        return builder.build();
    }
}
