/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.admin.impl;

import dk.nsi.fmk.admin.Admin;
import dk.nsi.fmk.admin.client.AdminClientFactory;
import dk.nsi.fmk.moduleframework.client.config.ClientApplicationConfig;
import org.springframework.beans.factory.annotation.Autowired;
import lombok.extern.log4j.Log4j2;
import org.springframework.core.env.Environment;


@Log4j2
public class AdminModuleClientFactory {
    private boolean useMock;
    private String admClientPropertyFile;

    @Autowired
    private Environment env;

    public AdminModuleClientFactory(boolean useMock) {
        this.useMock = useMock;
    }

    public Admin getAdminServiceClient() {

        // this file name could be dynamic (ie. defined using Spring config) rather than hardcoded!
        if (env.getProperty("spring.config.location") != null) {
            admClientPropertyFile = env.getProperty("spring.config.location")+"/admin-client.properties";
        }
        else {
            admClientPropertyFile = "admin-client.properties";
        }
        try {
            Admin admin;
            log.info("AdminModuleClientFactory.getAdminServiceClient called: useMock=" + useMock);
            if (useMock) {
                admin = AdminClientFactory.createAdminClientStub();
            } else {
                // create a connection to a remote host running the admin-service using the specified properties
                ClientApplicationConfig clientConfig = new ClientApplicationConfig(admClientPropertyFile);
                if (log.isInfoEnabled()) {
                    // alternative: use slf4j and set log level to INFO for "dk.nsi.fmk.moduleframework.client.config" to dump loaded properties
                    log.info("AdminModuleClientFactory loaded properties for AdminClient, property-path: " + admClientPropertyFile
                            + "\n" + clientConfig.getPropertiesAsString("pwd", "password", "passwd"));
                }
                admin = AdminClientFactory.createAdminClient(clientConfig);
                log.debug("AdminClient created using AdminClientFactory.createAdminClient with config from property file=" + admClientPropertyFile);
            }
            return admin;
        } catch (Exception e) {
            log.error("Could not create new AdminClient - got exception", e);
            throw e;
        }
    }
}
