/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.admin.impl;

public enum Role {
    Doctor(1, false, "Læge", "doctor"),
    Dentist(2, false, "Tandlæge", "dentist"),
    Midwife(3, false, "Jordemoder", "midwife"),
    Nurse(4, false, "Sygeplejerske", "nurse"),

    AssistantForDoctor(101, true, "Assistent for Læge", "assistant for doctor"),
    AssistantForDentist(102, true, "Assistent for Tandlæge", "assistant for dentist"),
    AssistantForMidwife(103, true, "Assistent for Jordemoder", "assistant for midwife"),
    AssistantForNurse(104, true, "Assistent for Sygeplejerske", "assistant for nurse"),

    PrescriptionRegistrator(56, false, "Recept registrator", "Prescription Registrator"),
    ECPRCoordinator(59, false, "eCPR koordinator", "eCPR coordinator"),
    WebAdmin(999, false, "Web administrator", "webadmin");

    private final int id;
    private final boolean isAssistantRole;
    private final String schemaName;
    private final String name;

    private Role(int id, boolean assistantRole, String schemaName, String name) {
        this.id = id;
        isAssistantRole = assistantRole;
        this.schemaName = schemaName;
        this.name = name;
    }

    public static Role forName(String name) {
        for (Role role : values()) {
            if (role.getName().equals(name)) {
                return role;
            }
        }
        return null;
    }

    public static Role forSchemaName(String name) {
        for (Role role : values()) {
            if (role.getSchemaName().equals(name)) {
                return role;
            }
        }
        return null;
    }

    public String getName() {
        return name;
    }

    public String getSchemaName() {
        return schemaName;
    }

    public long getID() {
        return id;
    }

    public static Role roleForEducationCode(String educationCode) {
        switch (educationCode) {
            case "7170":
                return Doctor;
            case "5433":
                return Dentist;
            case "5175":
                return Midwife;
            case "5166":
                return Nurse;
//            case "5152":
//                return SosuAssist;
//            case "B511":
//                return PharmacistWithPrescriptionRights;
            default:
                return null;
        }
    }
}
