/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.handler;

import dk.sds.ecpr2.domain.NameVO;
import dk.sds.ecpr2.domain.PersonVO;
import dk.sds.ecpr2.domain.RequestContextVO;
import dk.sds.ecpr2.domain.ServiceActorVO;
import org.apache.commons.text.similarity.EditDistance;
import org.apache.commons.text.similarity.JaccardSimilarity;
import org.apache.commons.text.similarity.LevenshteinDistance;
import org.apache.commons.text.similarity.SimilarityScore;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class CompareNameHandler extends AbstractHandler {
    public Integer execute(List<PersonVO> personList, String nameToCompare, ServiceActorVO actor, RequestContextVO context) {
        if (personList == null || personList.isEmpty()) {
            return null;
        }
        int highestScore = 0;
//        SimilarityScore<Double> similarityScore = new JaccardSimilarity();
        SimilarityScore<Integer> similarityScore = new LevenshteinDistance();
        Date now = new Date();
        for (PersonVO person : personList) {
            for (NameVO name : person.getNames()) {
                if (name.getValidTo().after(now)) {
                    int score;
                    if (similarityScore instanceof EditDistance) {
                        score = (int) Math.round((1-((double) similarityScore.apply(nameToCompare, name.getFullName())/Math.max(nameToCompare.length(), name.getFullName().length()))) * 100);
                    } else {
                        score = (int) Math.round(similarityScore.apply(nameToCompare, name.getFullName()) * 100);
                    }
                    if (score > highestScore) {
                        highestScore = score;
                    }
                }
            }
        }
        return highestScore;
    }

}
