/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.handler;

import dk.sds.ecpr2.Modified;
import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.*;
import dk.sds.ecpr2.webservice.validation.ValidationErrors;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@Component
public class DeletePersonHandler extends AbstractHandler {
    public void execute(List<IdentifierVO> identifiers, Modified modified, ServiceActorVO actor, RequestContextVO context) throws DAOException, IOException {
        validator.validateDeletePersonVersion(modified);
        if (identifiers == null || identifiers.isEmpty()) {
            return;
        }
        List<PersonVO> personWithRelations = getPersonById(identifiers);
        if (CollectionUtils.isEmpty(personWithRelations)) {
            throw new IllegalArgumentException(validator.format(ValidationErrors.KEY_IDENTIFIER_INVALID, identifiers.get(0).getValue()));
        }
        for (PersonVO person : personWithRelations) {
            if (person.getPersonUUID() != null) {
                for (AddressVO addressVO : person.getAddresses()) {
                    addressDAO.updateTimeStamp(addressVO, Date.from(context.getRequestTimeStamp()));
                }
                for (BirthDateVO birthDateVO : person.getBirthDates()) {
                    birthDateDAO.updateTimeStamp(birthDateVO, Date.from(context.getRequestTimeStamp()));
                }
                for (ContactVO contactVO : person.getContacts()) {
                    contactDAO.updateTimeStamp(contactVO, Date.from(context.getRequestTimeStamp()));
                }
                for (GenderVO genderVO : person.getGenders()) {
                    genderDAO.updateTimeStamp(genderVO, Date.from(context.getRequestTimeStamp()));
                }
                for (IdentifierVO identifierVO : person.getIdentifiers()) {
                    identifierDAO.updateTimeStamp(identifierVO, Date.from(context.getRequestTimeStamp()));
                }
                for (NameVO nameVO : person.getNames()) {
                    nameDAO.updateTimeStamp(nameVO, Date.from(context.getRequestTimeStamp()));
                }
                personDAO.deletePerson(person.getPersonUUID());
                searchPersonDAO.deleteIndex(Collections.singletonList(person));
            }
        }
    }

}
