/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.handler;

import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.PersonVO;
import dk.sds.ecpr2.domain.ServiceActorVO;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Component
public class GetPersonIdHandler extends AbstractHandler {
    public List<PersonVO> execute(List<PersonVO> persons, ServiceActorVO actor) throws DAOException {
        List<PersonVO> listOfPersons = new ArrayList<>();
        for (PersonVO personVO: persons) {
            try {
                validator.validateKeyIdentifier(personVO);
            } catch (IllegalArgumentException e) {
                continue;
            }
            List<PersonVO> personList = crossTableDAO.getPersonsWithRelations(Collections.singletonList(personVO.getPersonUUID()));
            PersonVO returnedPerson = mergePersons(personList);
            listOfPersons.add(returnedPerson);
        }

        return listOfPersons;
    }
}
