/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.handler;

import dk.sds.ecpr2.domain.PersonVO;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class SearchPersonHandler extends AbstractHandler {
    public List<PersonVO> execute(String searchStr) throws Exception {
        List<String> result = Arrays.asList(searchStr.split(" AND "));
        List<PersonVO> personList = searchPersonDAO.searchPerson(result);
        addModifiers(personList, false);
        Set<String> personUuids = personList.stream().map(PersonVO::getPersonUUID).collect(Collectors.toSet());
        Map<String, List<PersonVO>> personMap = new TreeMap<>();
        personUuids.forEach(personUuid -> personMap.put(personUuid, new ArrayList<PersonVO>()));
        personList.forEach(p -> personMap.get(p.getPersonUUID()).add(p));

        List<PersonVO> personsList = new ArrayList<>();
        for (List<PersonVO> list : personMap.values()) {
            personsList.add(mergePersons(list));
        }
        return personsList;
    }
}
