/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.historyExport;

import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

@Component
@EnableScheduling
@Log4j2
public class UploadToNSPJob {
    @Value("${jobs.uploadToNsp.enabled}")
    private boolean jobIsEnabled;

    private int errorCount = 0;
    private final int delayWhenErrorMs = 10_000;
    private final int maximumWaitTimeMs = 5 * 60 * 1000;

    @Autowired
    protected UploadToNSPServiceImpl uploadToNSPService;

    @Scheduled(initialDelayString = "${jobs.uploadToNSP.initialDelay:3600000}", fixedDelayString = "${jobs.uploadToNSP.delayBetween:300000}")
    public void runUploadToNSP() throws InterruptedException {
        if ( ! jobIsEnabled) {
            log.error("Upload to NSP job is disabled");
            return;
        }
        try {
            //Call Method
            uploadToNSPService.uploadToNSP();
            errorCount = 0;
        } catch (Exception e) {
            errorCount++;
            log.error("Export history not executed because of an error.", e);
        }  finally {
            // It probably does not help to try again immediately. So wait longer for every error.
            Thread.sleep(Math.min(errorCount * delayWhenErrorMs, maximumWaitTimeMs));
        }
    }
}
