/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.util;

public enum FaultCode {

    GENERAL_ERROR(100, "Der opstod en fejl"),

    GET_ECPR_VALIDATION_ERROR(110, "Fejl i request ifm. hentning af eCPR-data"),
    GET_ECPR_INTERNAL_ERROR(111, "Intern fejl ifm. hentning af eCPR-data"),

    CREATE_ECPR_VALIDATION_ERROR(120, "Fejl i request ifm. oprettelse af eCPR-data"),
    CREATE_ECPR_INTERNAL_ERROR(121, "Intern fejl ifm. oprettelse af eCPR-data"),

    UPDATE_ECPR_VALIDATION_ERROR(130, "Fejl i request ifm. opdatering af eCPR-data"),
    UPDATE_ECPR_INTERNAL_ERROR(130, "Intern fejl ifm. opdatering af eCPR-data"),

    DELETE_ECPR_VALIDATION_ERROR(140, "Fejl i request ifm. sletning af eCPR-data"),
    DELETE_ECPR_INTERNAL_ERROR(140, "Intern fejl ifm. sletning af eCPR-data"),

    BUILD_LUCENE_INDEX_ERROR(150, "Intern fejl ifm. oprettelse / opdatering af Lucene Index"),
    BUILD_LUCENE_INDEX_INTERNAL_ERROR(160, "Intern fejl ift. oprettelse / opdatering af Lucene Index"),

    SEARCH_PERSON_INTERNAL_SERVICE_ERROR(170, "Intern fejl i forbindelse med søgning af person"),
    SEARCH_PERSON_VALIDATION_ERROR(180, "Fejl i request ifm. søgning af person"),
    SEARCH_PERSON_INTERNAL_ERROR(190, "Intern fejl ifm. søgning af en person"),

    CREATE_PERSON_INTERNAL_SERVER_ERROR(201, "Intern fejl i forbindelse med oprettelse"),
    CREATE_NEW_ECPR_ERROR(202, "Der opstod en fejl i forbindelse med generering af nyt eCPR-nummer"),

    UPDATE_PERSON_INTERNAL_SERVER_ERROR(211, "Intern fejl i forbindelse med opdatering"),
    UPDATE_PERSON_LUCENE_INDEX_INTERNAL_SERVER_ERROR(212, "Intern fejl i forbindelse med opdatering af person i lucene index"),
    CREATE_PERSON_LUCENE_INDEX_INTERNAL_SERVER_ERROR(213, "Intern fejl i forbindelse med opdatering af lucene index ved oprettelse af person"),

    GET_PERSON_BY_PERSONID_ERROR(220, "Person eksisterer ikke"),
    GET_PERSON_BY_PERSONID_VALIDATION_ERROR(230, "Fejl i request ifm. opslag af person med personId"),
    GET_PERSON_BY_PERSONID_INTERNAL_ERROR(240, "Intern fejl ifm. opslag af person med personId"),

    GET_PERSON_BY_PERSONID_INTERNAL_SERVER_ERROR(250, "Intern fejl i forbindelse med indlæsning af person med personId"),

    DELETE_PERSON_INTERNAL_SERVER_ERROR(260, "Intern fejl i forbindelse med sletning af person"),
    DELETE_PERSON_LUCENE_INDEX_INTERNAL_SERVER_ERROR(270, "Intern fejl i forbindelse med sletning af person i Lucene Index"),
    DELETE_PERSON_BY_PERSONID_VALIDATION_ERROR(270, "Fejl i request ifm. sletning af person"),
    DELETE_PERSON_BY_PERSONID_INTERNAL_ERROR(280, "Intern fejl ifm. sletning af person"),
    DELETE_PERSON_BY_PERSONID_NOT_ALLOWED_IN_PROD(290, "Sletning af person er ikke tilladt"),

    CREATE_PERSONID_INTERNAL_SERVER_ERROR(300, "Intern fejl i forbindelse med oprettelse af nyt eCPR"),
    CREATE_PERSONID_VALIDATION_ERROR(310, "Fejl i request ifm. oprettelse af nyt eCPR"),
    CREATE_PERSONID_INTERNAL_ERROR(320, "Intern fejl ifm. oprettelse af nyt eCPR"),

    MERGE_PERSON_INTERNAL_SERVER_ERROR(400, "Intern fejl i forbindelse med fletning af person"),
    MERGE_PERSON_VALIDATION_ERROR(420, "Fejl i request ifm. fletning af personer"),
    MERGE_PERSON_INTERNAL_ERROR(430, "Intern fejl ifm. fletning af personer"),

    AUTHORIZATION_ERROR_INVALID_HEADER(500, "DGWS/IDWS headers er ikke gyldige."),
    AUTHORIZATION_ERROR(501, "Adgang ikke tilladt."),
    AUTHORIZATION_INTERNAL_ERROR(502, "Intern serverfejl ifm. check af adgangsrettigheder"),

    EXPORT_HISTORY_INTERNAL_SERVER_ERROR(550, "Intern fejl ifm. indsættelse af data ifm. data eksportering"),

    FECTH_DATABASECHANGES_TO_EXPORT_INTERNAL_ERROR(600, "Intern fejl i ifm. indlæsning af data til exportering"),

    GET_PERMISSIONS_INTERNAL_SERVER_ERROR(700, "Intern fejl i forbindelse med hentning af rettigheder"),
    GET_PERMISSIONS_VALIDATION_ERROR(710, "Fejl i request ifm. hentning af rettigheder"),
    GET_PERMISSIONS_INTERNAL_ERROR(720, "Intern fejl ifm. hentning af rettigheder"),

    GET_OIDS_INTERNAL_SERVER_ERROR(800, "Intern fejl i forbindelse med hentning af OIDer"),
    GET_OIDS_VALIDATION_ERROR(810, "Fejl i request ifm. hentning af OIDer"),
    GET_OIDS_INTERNAL_ERROR(820, "Intern fejl ifm. hentning af OIDer"),

    GET_CPR_PERSON_INTERNAL_SERVER_ERROR(900, "Intern fejl i forbindelse med hentning af cpr person"),
    GET_CPR_PERSON_VALIDATION_ERROR(910, "Fejl i request ifm. hentning af cpr person"),
    GET_CPR_PERSON_INTERNAL_ERROR(920, "Intern fejl ifm. hentning af cpr person"),
    GET_CPR_PERSON_STAMDATA_ERROR(930, "Fejl i kald til CPR Stamdata"),

    COMPARE_NAME_INTERNAL_SERVER_ERROR(1000, "Intern fejl i forbindelse med sammenligning af navne"),
    COMPARE_NAME_VALIDATION_ERROR(1010, "Fejl i request ifm. sammenligning af navne"),
    COMPARE_NAME_INTERNAL_ERROR(1020, "Intern fejl ifm. sammenligning af navne");


    private final int code;
    private final String message;

    public int getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    FaultCode(int code, String message) {
        this.code = code;
        this.message = message;
    }

    @Override
    public String toString() {
        return code + ": " + message;
    }
}
