/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.util;

import lombok.extern.log4j.Log4j2;

import java.io.PrintWriter;
import java.io.StringWriter;

@Log4j2
public class FaultUtil {

    public static void throwECPR2Fault(FaultCode faultCode) {
        throw new ECPRException(faultCode);
    }

    public static void throwECPR2Fault(FaultCode faultCode, Throwable cause) {
        throw new ECPRException(faultCode, cause);
    }

    public static void throwECPR2Fault(FaultCode faultCode, String faultInfo) {
        throw new ECPRException(faultCode, faultInfo, null);
    }

    public static void throwFault(FaultCode faultCode, Throwable t) throws Exception {
        throwFault(faultCode, null, t);
    }

    public static void throwFault(FaultCode faultCode, String faultInfo, Throwable cause) {
        String message = faultCode.toString() + (faultInfo != null ? ", " + faultInfo : "");
        log.error(message, cause);
        throw new ECPRException(faultCode, faultInfo, cause);
    }

}
