/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2.util.ECPRException;
import dk.sds.ecpr2.util.FaultCode;
import dk.sds.ecpr2.util.FaultUtil;
import dk.sds.ecpr2.webservice.exceptions.ParseException;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class AbstractWebService {

    protected void commonExceptionHandler(Exception e, FaultCode validationErrorCode, FaultCode internalServerErrorCode) throws Exception {
        log.warn(e);
        e.printStackTrace();
        if (e instanceof IllegalArgumentException || e instanceof ParseException) {
            FaultUtil.throwFault(validationErrorCode, e.getMessage(), e);
        } else if (e instanceof ECPRException) {
            final ECPRException ecprEx = (ECPRException) e;
            FaultUtil.throwFault(ecprEx.getFaultCode(), ecprEx.getMessage(), ecprEx);
        }

        FaultUtil.throwFault(internalServerErrorCode, e);
    }
}
