/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2._2017._10._01.GetOIDsRequest;
import dk.sds.ecpr2._2017._10._01.GetOIDsResponse;
import dk.sds.ecpr2._2017._10._01.ObjectFactory;
import dk.sds.ecpr2.domain.OIDVO;
import dk.sds.ecpr2.service.ECPR2Service;
import dk.sds.ecpr2.util.FaultCode;
import dk.sds.ecpr2.webservice.mapping.ToXMLMapper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;

import java.util.List;


@Endpoint
@Log4j2
public class ECPR2MasterDataWebServiceImpl extends AbstractWebService implements ECPR2MasterDataWebService {

    @Autowired
    private ECPR2Service businessService;

    @Autowired
    ToXMLMapper responseMapper;

    @Override
    @ResponsePayload
    public GetOIDsResponse GetOIDs(@RequestPayload final GetOIDsRequest request) throws Exception {
        final GetOIDsResponse response = new ObjectFactory().createGetOIDsResponse();
        try {
            if (request != null) {
                List<OIDVO> list = businessService.getOIDs();
                response.getOIDS().addAll(responseMapper.mapOIDs(list));
            }
        } catch (Exception e) {
            commonExceptionHandler(e, FaultCode.GENERAL_ERROR, FaultCode.GENERAL_ERROR);
        }
        return response;
    }

}
