/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;


import com.fasterxml.jackson.databind.ObjectMapper;
import dk.sds.ecpr2.domain.BlacklistedDeCPRVO;
import dk.sds.ecpr2.domain.BlacklistingResponseVO;
import dk.sds.ecpr2.domain.BlacklistingVO;
import dk.sds.ecpr2.service.ECPR2Service;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Log4j2
@Controller
@RequestMapping("/blacklisting/v1")
public class ECPRRestService {

    @Autowired
    ECPR2Service service;

    @PutMapping("")
    public ResponseEntity putBlacklisting(@RequestBody BlacklistingVO blacklisting) {
         try {
             if (requestIsInvalid(blacklisting)) return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();

             boolean alreadyBlacklisted = false;
             for (String deCPR : blacklisting.getDeCprs()) {
                 if (!service.isBlacklistedDeCPR(blacklisting.getOid(), deCPR).getOids().isEmpty()) {
                     alreadyBlacklisted = true;
                 }
                 service.blacklistDeCPR(blacklisting.getOid(), deCPR);
             }

             if (alreadyBlacklisted) {
                 return ResponseEntity.ok().build();
             } else {
                return ResponseEntity.created(new URI("")).build();
             }
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
        }
    }

    @PostMapping("")
    public ResponseEntity<String> postBlacklisting(@RequestBody BlacklistingVO blacklisting) {
        try {
            if (requestIsInvalid(blacklisting)) return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();

            BlacklistingResponseVO alreadyBlacklisted = new BlacklistingResponseVO();
            for (String deCPR : blacklisting.getDeCprs()) {
                BlacklistedDeCPRVO blacklistedDeCPRVO = service.isBlacklistedDeCPR(blacklisting.getOid(), deCPR);
                if (!blacklistedDeCPRVO.getOids().isEmpty()) {
                    alreadyBlacklisted.getBlacklist().add(blacklistedDeCPRVO);
                } else {
                    service.blacklistDeCPR(blacklisting.getOid(), deCPR);
                }
            }

            if (!alreadyBlacklisted.getBlacklist().isEmpty()) {
                ObjectMapper objectMapper = new ObjectMapper();
                return ResponseEntity.status(HttpStatus.CONFLICT).body(objectMapper.writeValueAsString(alreadyBlacklisted));
            } else {
                return ResponseEntity.created(new URI("")).build();
            }
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
        }
    }

    private boolean requestIsInvalid(BlacklistingVO blacklisting) {
        if (blacklisting == null || blacklisting.getDeCprs().size() == 0 || blacklisting.getOid() == null) {
            return true;
        }
        return false;
    }

}
