/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao;

import dk.sds.ecpr2.OID;
import dk.sds.ecpr2.domain.IdentifierVO;
import dk.sds.ecpr2.domain.PersonVO;
import dk.sds.ecpr2.domain.ServiceActorVO;
import dk.sds.ecpr2.util.TimeHelper;

import java.util.Calendar;
import java.util.Date;
import java.util.UUID;

public class DaoTestHelper {
    public static final String PERSON_UUID_FROM_DATA_FILL = "b90622d4-f088-11e7-8c3f-9a214cf093ae";

    public static IdentifierVO buildTestIdentifierVO(Integer personPID, String identifierValue) {
        IdentifierVO beforeTest = new IdentifierVO();
        beforeTest.setPersonPID(personPID);
        beforeTest.setUuid(UUID.randomUUID().toString());
        beforeTest.setValue(identifierValue);
        beforeTest.setOid(OID.C_ECPR);
        beforeTest.setValidity(16);
        beforeTest.setExpiry(TimeHelper.createNew(2017, Calendar.DECEMBER, 31));
        beforeTest.setValidFrom(TimeHelper.createNew(2017, Calendar.DECEMBER, 31));
        beforeTest.setValidTo(TimeHelper.createNew(2030, Calendar.DECEMBER, 31));
        return beforeTest;
    }

    public static PersonVO buildTestPersonVO(String uuid, int serviceActorPID) {
        PersonVO p = new PersonVO();
        p.setPersonUUID(uuid);
        p.setServiceActorPID(serviceActorPID);
        p.setVersion(1);
        p.setVersionDescription("ÆØÅæøå");
        p.setModified(new Date());
        p.setReservedUntil(TimeHelper.createNew(2018, Calendar.JANUARY, 31));
        return p;
    }

    public static ServiceActorVO buildTestServiceActorVO(String personIdentifier) {
        final String personIdentifierOID = OID.CPR;
        final String firstName = "Broder";
        final String lastName = "Salsa";
        final String role = "Rolle";
        final String authorisationNumber = "123A4";
        final String organizationId = "123456";
        final String getOrganizationName = "AUH";
        final String regionIdentifierSource = "1.2.208.176.1.1002";

        final ServiceActorVO sa = new ServiceActorVO();
        //sa.setServiceActorPID(0);
        sa.setPersonIdentifier(personIdentifier);
        sa.setPersonIdentifierOID(personIdentifierOID);
        sa.setFirstName(firstName);
        sa.setLastName(lastName);
        sa.setRole(role);
        sa.setModifiedTime(new Date());
        sa.setAuthorisationNumber(authorisationNumber);
        sa.setOrganizationId(organizationId);
        sa.setOrganizationName(getOrganizationName);
        //sa.setHashValue(0);
        //sa.setModifiedTime(new Date());

        return sa;
    }


}
