/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao.impl;

import dk.sds.ecpr2.IntegrationTest;
import dk.sds.ecpr2.domain.PersonVO;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.RAMDirectory;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class LuceneIT extends IntegrationTest {
    protected static final String PERSONUUID = "b90622d4-f088-11e7-8c3f-9a214cf093ae";

    @Autowired
    SearchCrossTableDAOImpl crossTableDAO;

    @Autowired
    SearchPersonDAOImpl searchPersonDAO;

    private RAMDirectory ramDirectory;

    @Before
    public void setUpIndex() throws Exception {
        if (ramDirectory == null) {
            searchPersonDAO.setCrossTableDAO(crossTableDAO);
            ramDirectory = new RAMDirectory();

            long startTime = System.currentTimeMillis();
            searchPersonDAO.buildIndex(null, new IndexWriter(ramDirectory, new IndexWriterConfig(new StandardAnalyzer())), false);
            long endTime = System.currentTimeMillis();
            long duration = (endTime - startTime);
            System.out.println("\n ############## BUILD INDEX DURATION: " + duration / 1000 + " SECOND #################");
        }
    }

    @Ignore
    @Test // Kræver at data fra generatoren er indlæst. Dette kan inteferere med andre tests
    public void searchPerson_ByPersonUUID_PersonFound() throws Exception {
        String s = "familyname:FamilyName3394";
        List<String> result = Arrays.asList(s.split("AND"));

        IndexReader reader = DirectoryReader.open(ramDirectory);
        long startTime = System.currentTimeMillis();
        List<PersonVO> personList = searchPersonDAO.searchPerson(new IndexSearcher(reader), result);
        long endTime = System.currentTimeMillis();
        long duration = (endTime - startTime);
        System.out.println("\n############## SEARCH PERSON DURATION: " + duration / 1000 + " SECOND #################");
        assertNotEquals(0, personList.size());
        int size = personList.size();
        assertEquals(237, personList.size());

    }
}
