/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.dao.impl;

import dk.sds.ecpr2.DatabaseTest;
import dk.sds.ecpr2.dao.DaoTestHelper;
import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.PersonVO;
import dk.sds.ecpr2.util.TimeHelper;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import static org.junit.Assert.*;


public class PersonDAOImplTest extends DatabaseTest {

    @Autowired
    private PersonDAOImpl dao;

    @Test
    public void createAndGetPerson_CreatesCorrectData() throws DAOException {
        final String uuid = UUID.randomUUID().toString();
        PersonVO beforeTest = DaoTestHelper.buildTestPersonVO(uuid, 1);
        dao.createPersons(Arrays.asList(beforeTest));
        List<PersonVO> result = dao.getPersonsByUUIDs(Arrays.asList(uuid));

        assertNotNull(result);
        assertEquals(1, result.size());
        PersonVO afterTest = result.get(0);
        assertEquals(beforeTest.getPersonUUID(), afterTest.getPersonUUID());
        assertEquals(beforeTest.getVersion(), afterTest.getVersion());
        assertEquals(beforeTest.getVersionDescription(), afterTest.getVersionDescription());
        assertTrue(beforeTest.getReservedUntil().equals(afterTest.getReservedUntil()));
        assertTrue(afterTest.getModifiedTime().before(TimeHelper.getAfterCurrentTime(1)));
    }

    @Test
    public void createPersonsBatch_InsertsRightNumberOfRows() throws DAOException {
        final PersonVO Person1 = DaoTestHelper.buildTestPersonVO(UUID.randomUUID().toString(), 1);
        final PersonVO Person2 = DaoTestHelper.buildTestPersonVO(UUID.randomUUID().toString(), 1);

        int rowCountBefore = dao.rowCount();
        dao.createPersons(Arrays.asList(Person1, Person2));

        int rowCountAfter = dao.rowCount();
        assertEquals(2, rowCountAfter - rowCountBefore);
    }


}
