/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;

import dk.sds.ecpr2.UnitTest;
import org.junit.Test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class CountryCodeTests extends UnitTest {

    @Test
    public void isFullNameValid_WhenExists() {
        assertTrue(CountryCode.isFullNameValid("Danmark"));
    }

    @Test
    public void isFullNameValid_WhenMissing() {
        assertFalse(CountryCode.isFullNameValid("NotACountry"));
    }

    @Test
    public void isTwoLetterCodeValid_WhenExists() {
        assertTrue(CountryCode.isTwoLetterAbbreviationValid("DK"));
    }

    @Test
    public void isTwoLetterCodeValid_WhenMissing() {
        assertFalse(CountryCode.isTwoLetterAbbreviationValid("XX"));
    }

    @Test
    public void isThreeLetterCodeValid_WhenExists() {
        assertTrue(CountryCode.isThreeLetterAbbreviationValid("DNK"));
    }

    @Test
    public void isThreeLetterCodeValid_WhenMissing() {
        assertFalse(CountryCode.isThreeLetterAbbreviationValid("XXX"));
    }

    @Test
    public void isNumericValueValid_WhenExists() {
        assertTrue(CountryCode.isNumericValueValid(208));
    }

    @Test
    public void isNumericValueValid_WhenMissing() {
        assertFalse(CountryCode.isNumericValueValid(999));
    }


}
