/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.ExportHistory;

import com.github.stefanbirkner.fakesftpserver.rule.FakeSftpServerRule;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.SftpException;
import dk.sds.ecpr2.PersonType;
import dk.sds.ecpr2.service.historyExport.*;
import dk.sds.ecpr2.webservice.WebServiceTestAbstract;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.io.IOException;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

public class UploadToNSPImplTest extends WebServiceTestAbstract {

    @Value("${sftp.remote.path}")
    private String destination;

    @Value("${temp.dir}")
    private String tmpDir;

    @Autowired
    ExportHistoryServiceImpl ecpr2ExportService;
    @Autowired
    UploadToNSPServiceImpl uploadToNSPService;

    private static final String TEST_USER = "test";
    private static final String TEST_PASSWORD = "Test1234";
    private static final int TEST_PORT = 22999;

    private Instant startTime = null;

    @Autowired
    NspManagerSFtp nspManager;

    @Rule
    public final FakeSftpServerRule fakeSftpServerRule = new FakeSftpServerRule()
            .setPort(TEST_PORT)
            .addUser(TEST_USER, TEST_PASSWORD);


    @Before
    public void onSetUp() throws Exception {
        Instant startTime = Instant.now().minusSeconds(1);
        this.startTime = startTime;
        createPersons();

        nspManager.sftpClient = createSFtpClient();

//        SFTPTestServer.startServer(TEST_USER, TEST_PASSWORD, TEST_PORT);

    }

    @After
    public void tearDown() throws Exception {
        //FileUtils.forceDelete(new File(this.destination));
//        SFTPTestServer.destroy();
    }

    @Test
    public void uploadToNsp_testUpload() throws SftpException, JSchException, InterruptedException, IOException {
        fakeSftpServerRule.createDirectory(destination);
        File exportedFile = ecpr2ExportService.exportHistoryToFiles(null, 1);
        String exportedFileName = exportedFile.listFiles()[0].getName();
        uploadToNSPService.uploadToNSP();
        fakeSftpServerRule.existsFile(destination + "/" + exportedFileName);
    }

    private void createPersons() throws Exception {
        List<PersonType> createdPersons = new ArrayList<>();
        PersonType p1 = createPerson("Benny", "Olsen", "Testvej 16", "Testby", "8000", "1981-05-19", "DK");
        PersonType p2 = createPerson("Benny", "Olsen", "Testvej 16", "Testby", "8000", "1981-05-19", "DK");
        createdPersons.add(p1);
        createdPersons.add(p2);
        List<PersonType> persons = createPersonsInDb(createdPersons);
    }

    private SFtpClient createSFtpClient() {
        SFtpClientImpl sFtpClient = new SFtpClientImpl();
        sFtpClient.user = TEST_USER;
        sFtpClient.password = TEST_PASSWORD;
        sFtpClient.port = TEST_PORT;
        sFtpClient.hostName = "localhost";

        return sFtpClient;
    }
}
