/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2.IntegrationTest;
import dk.sds.ecpr2.domain.BlacklistingVO;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import java.util.Arrays;

import static org.junit.Assert.assertEquals;

public class ECPRRestServiceTest extends IntegrationTest {

    private static final String VALID_USERNAME = "TestUser";
    private static final String VALID_PASSWORD = "Password";
    private static final String INVALID_USERNAME = "UnknownUser";
    private static final String INVALID_PASSWORD = "InvalidPassword";
    public static final String PUT_URL = "/blacklisting/v1";
    public static final String POST_URL = "/blacklisting/v1";

    @Autowired
    TestRestTemplate testRestTemplate;

    @Ignore
    @Test
    public void putBlacklisting_WithValidInput_ReturnsCreatedReponse() {
        final ResponseEntity response = testRestTemplate.withBasicAuth(VALID_USERNAME, VALID_PASSWORD).exchange(PUT_URL, HttpMethod.PUT, getValidBlacklistingRequest(), String.class);
        assertEquals(HttpStatus.CREATED, response.getStatusCode());
    }

    @Ignore
    @Test
    public void postBlacklisting_WithValidInput_ReturnsCreatedReponse() {
        final ResponseEntity response = testRestTemplate.withBasicAuth(VALID_USERNAME, VALID_PASSWORD).exchange(POST_URL, HttpMethod.POST, getValidBlacklistingRequest(), String.class);
        assertEquals(HttpStatus.CREATED, response.getStatusCode());
    }

    @Ignore
    @Test
    public void putBlacklisting_WithValidInputButUnknownUserName_ReturnsUnauthorizedReponse() {
        final ResponseEntity response = testRestTemplate.withBasicAuth(INVALID_USERNAME, VALID_PASSWORD).exchange(PUT_URL, HttpMethod.PUT, getValidBlacklistingRequest(), String.class);
        assertEquals(HttpStatus.UNAUTHORIZED, response.getStatusCode());
    }

    @Ignore
    @Test
    public void putBlacklisting_WithValidInputButInvalidPassword_ReturnsUnauthorizedReponse() {
        final ResponseEntity response = testRestTemplate.withBasicAuth(VALID_USERNAME, INVALID_PASSWORD).exchange(PUT_URL, HttpMethod.PUT, getValidBlacklistingRequest(), String.class);
        assertEquals(HttpStatus.UNAUTHORIZED, response.getStatusCode());
    }

    private HttpEntity<BlacklistingVO> getValidBlacklistingRequest() {
        return new HttpEntity<>(new BlacklistingVO("oid", Arrays.asList("cpr1", "cpr2")));
    }
}
