/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2.PredefinedOIDType;
import dk.sds.ecpr2._2017._10._01.GetOIDsRequest;
import dk.sds.ecpr2._2017._10._01.GetOIDsResponse;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.stream.Collectors;

import static org.junit.Assert.assertEquals;

@RunWith(SpringRunner.class)
public class MasterDataTest extends WebServiceTestAbstract {

    @Test
    public void testLookup() throws Exception {
        GetOIDsRequest request = new GetOIDsRequest();
        GetOIDsResponse getOIDsResponse = masterDataservice.GetOIDs(request);
        assertEquals(13, getOIDsResponse.getOIDS().size());

        assertEquals(13, getOIDsResponse.getOIDS().stream().filter(oid -> oid.getOID() != null).collect(Collectors.toList()).size());
        assertEquals(13, getOIDsResponse.getOIDS().stream().filter(oid -> oid.getPrimaryLabel() != null).collect(Collectors.toList()).size());
        assertEquals(1, getOIDsResponse.getOIDS().stream().filter(oid -> PredefinedOIDType.CPR_NUMMER.equals(oid.getType())).collect(Collectors.toList()).size());
        assertEquals(1, getOIDsResponse.getOIDS().stream().filter(oid -> "Region Syddanmark".equals(oid.getSecondaryLabel())).collect(Collectors.toList()).size());
        assertEquals(1, getOIDsResponse.getOIDS().stream().filter(oid -> PredefinedOIDType.E_CPR_NUMMER.equals(oid.getType())).collect(Collectors.toList()).size());
        assertEquals(6, getOIDsResponse.getOIDS().stream().filter(oid -> PredefinedOIDType.E_CPR_NUMMER_OG_KILDE.equals(oid.getType())).collect(Collectors.toList()).size());
        assertEquals(1, getOIDsResponse.getOIDS().stream().filter(oid -> PredefinedOIDType.EKSTERN_PERSON_IDENTIFIKATION.equals(oid.getType())).collect(Collectors.toList()).size());
        assertEquals(4, getOIDsResponse.getOIDS().stream().filter(oid -> PredefinedOIDType.EKSTERN_PERSON_IDENTIFIKATION_OG_LAND.equals(oid.getType())).collect(Collectors.toList()).size());
        assertEquals(3, getOIDsResponse.getOIDS().stream().filter(oid -> oid.getRegEx() != null).collect(Collectors.toList()).size());
    }

}
