/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2.PersonType;
import dk.sds.ecpr2._2017._10._01.SearchPersonRequest;
import dk.sds.ecpr2._2017._10._01.SearchPersonResponse;
import dk.sds.ecpr2.dao.impl.SearchCrossTableDAOImpl;
import dk.sds.ecpr2.dao.impl.SearchPersonDAOImpl;
import dk.sds.ecpr2.domain.PersonVO;
import dk.sds.ecpr2.domain.RequestContextVO;
import dk.sds.ecpr2.domain.ServiceActorVO;
import dk.sds.ecpr2.service.ECPR2Service;
import dk.sds.ecpr2.webservice.mapping.FromXMLMapper;
import dk.sds.ecpr2.webservice.validation.ValidationErrors;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigInteger;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;


public class
SearchPersonTest extends WebServiceTestAbstract {
    @Autowired
    FromXMLMapper requestMapper;

    @Autowired
    SearchPersonDAOImpl searchPersonDAO;
    @Autowired
    SearchCrossTableDAOImpl crossTableDAO;
    @Autowired
    private ECPR2Service businessService;

    List<PersonType> personTypelist = new ArrayList<>();
    List<PersonVO> personVOlist = new ArrayList<>();

    @Before
    public void setUpIndex() throws Exception {
        Instant now = Instant.now().minusSeconds(10);
        searchPersonDAO.setCrossTableDAO(crossTableDAO);
        createTestData();
        for (PersonType person : personTypelist) {
            personVOlist.add(requestMapper.mapPersonFromRequest(person));
        }
        ServiceActorVO actor = new ServiceActorVO();
        for (PersonVO persons : personVOlist) {
            RequestContextVO requestContextVO = new RequestContextVO();
            requestContextVO.setRequestTimeStamp(Instant.now());
            businessService.createPerson(persons, actor, requestContextVO);
        }
        searchPersonDAO.buildIndex(now);
    }

    public void createTestData() {
        personTypelist.add(createPerson("Jacob", "Rasmussen", "Banegårdspladsen 16", "Aarhus", "8000", "1981-05-19", "DK"));
        personTypelist.add(createPerson("Kasper", "Larsen", "Banegårdspladsen 17", "Aarhus", "8260", "1982-05-19", "DK"));
        personTypelist.add(createPerson("Anders", "Larsen", "Munkegade 18", "København", "1000", "1984-06-19", "DK"));
        personTypelist.add(createPerson("Alan", "Olsen", "Banegårdspladsen 19", "København", "1100", "1982-07-19", "DK"));
        personTypelist.add(createPerson("Alain", "Deutsch", "Sønder Alle 16", "Gading", "8030", "1980-04-01", "DE"));
        personTypelist.add(createPerson("Dennis", "Hansen", "Albion Road 17", "Kolding", "8030", "1988-04-02", "DE"));
        personTypelist.add(createPerson("Alexander", "Larsen", "Albion Street 18", "Aarhus", "8120", "1988-06-02", "DK"));
        personTypelist.add(createPerson("Kasper", "Brødby", "Albion Road 19", "Skanderborg", "8120", "1991-07-19", "DK"));
        personTypelist.add(createPerson("Alexandar", "Tan Hansen", "Albion Road 19", "Skanderborg", "8120", "1991-07-19", "DK"));
    }

    @Ignore
    @Test
    public void testGenderValue() throws Exception {
        assertEquals(9, searchResultsForQuery("gender:male"));
    }

    @Test
    public void testTooManyResults() throws Exception {
        SearchPersonRequest request = new SearchPersonRequest();
        request.setQuery("gender:male");
        request.setMaxNumberOfResults(BigInteger.valueOf(1));
        SearchPersonResponse response = service.searchPerson(request);
        assertNotNull(response);
        assertNotNull(response.getPersons());
        assertEquals(0, response.getPersons().size());
    }

    @Test
    public void testQueryHits() throws Exception {
        //assertEquals(1, searchResultsForQuery("givenname:Ja   "));
        assertEquals(2, searchResultsForQuery("givenname:Ala"));
        assertEquals(2, searchResultsForQuery("givenname:Alexand"));
        assertEquals(1, searchResultsForQuery("givenname:\"Anders\""));
        assertEquals(0, searchResultsForQuery("givenname:Dennis AND country:DK"));
        assertEquals(2, searchResultsForQuery("givenname:Kasper AND country:DK"));
        assertEquals(1, searchResultsForQuery("givenname:Jacob AND country:DK AND postalcode:8000"));
        assertEquals(1, searchResultsForQuery("givenname:Anders AND familyname:Larsen"));
        assertEquals(1, searchResultsForQuery("givenname:Dennis AND country:DE"));
        assertEquals(1, searchResultsForQuery("givenname:Alain"));
        //assertEquals(1, searchResultsForQuery("givenname:J*cob"));
//        assertEquals(1, searchResultsForQuery("givenname:J?cob"));
        /*assertEquals(1, searchResultsForQuery("givenname:\"J*cob\""));*/
        assertEquals(1, searchResultsForQuery("familyname:Deutsch"));
        // Client also uses Larsen
        /*assertEquals(3, searchResultsForQuery("familyname:Larsen"));*/
        assertEquals(1, searchResultsForQuery("familyname:Brødby"));
        //assertEquals(3, searchResultsForQuery("country:DE"));
        assertEquals(4, searchResultsForQuery("streetaddress:Albi*"));
        assertEquals(1, searchResultsForQuery("streetaddress:\"Banegårdspladsen 16\""));
        assertEquals(1, searchResultsForQuery("streetaddress:\"Sønder Alle 16\""));
        assertEquals(1, searchResultsForQuery("birthdate:1988-04-02"));
        assertEquals(2, searchResultsForQuery("birthdate:1988"));
        assertEquals(2, searchResultsForQuery("birthdate:1991-07"));
        assertEquals(3, searchResultsForQuery("birthdate:*-07-19"));
        assertEquals(1, searchResultsForQuery("givenname:Den* AND postalcode:8030"));
        assertEquals(1, searchResultsForQuery("familyname:\"Tan Hansen\""));
    }

    @Test
    public void searchPerson_InvalidQueries_InvalidQuery() throws Exception {
        assertBadQuery("givenname:"); // Value must be provided
        assertBadQuery(":Benny"); // Key must be provided
        assertBadQuery("givengame:Benny"); // Key must be known
        assertBadQuery("givenname:Benny OR country:Tyskland"); // OR is forbidden
        assertBadQuery("givenname:Benny AND count*ry:Danmark"); // Wildcard in key
        assertBadQuery("givenname:Benny country:DK"); // Missing AND clause
        assertBadQuery("givenname:Bennycountry:DK"); // Bad formatting
        assertEmptyQuery("");
    }

    @Test
    public void searchPerson_EmptyQuery() throws Exception {
        assertEmptyQuery("");
    }

    private void assertBadQuery(String s) throws Exception {
        SearchPersonRequest request = new SearchPersonRequest();
        try {
            request.setQuery(s);
            service.searchPerson(request);
            fail("Expected should have occured for this query: " + request.getQuery() + ": " + ValidationErrors.SEARCH_BAD_QUERY);
        } catch (Exception e) {
            assertMessageContainsError(e.getMessage(), ValidationErrors.SEARCH_BAD_QUERY);
        }
    }

    private void assertEmptyQuery(String s) throws Exception {
        SearchPersonRequest request = new SearchPersonRequest();
        try {
            request.setQuery(s);
            service.searchPerson(request);
            fail("Expected should have occured for this query: " + request.getQuery() + ": " + ValidationErrors.QUERY_REQUIRED);
        } catch (Exception e) {
            assertMessageContainsError(e.getMessage(), ValidationErrors.QUERY_REQUIRED);
        }
    }

    private int searchResultsForQuery(String s) throws Exception {
        SearchPersonRequest request = new SearchPersonRequest();
        request.setQuery(s);
        SearchPersonResponse response = service.searchPerson(request);
        assertNotNull(response);
        assertNotNull(response.getPersons());
        return response.getPersons().size();
    }
}
