/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice.mapping;

import dk.sds.ecpr2.BirthDateType;
import dk.sds.ecpr2.UnitTest;
import dk.sds.ecpr2.domain.BirthDateVO;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

public class FromXMLMapperTest extends UnitTest {

    private final FromXMLMapper mapper = new FromXMLMapper();

    @Test
    public void testBirthDateMapping() {
        BirthDateType birthDate = new BirthDateType();
        birthDate.setValue("2018-01-23");
        BirthDateVO birthDateVO = mapper.mapBirthDate(birthDate);
        assertEquals(Integer.valueOf(2018), birthDateVO.getYear());
        assertEquals(Integer.valueOf(1), birthDateVO.getMonth());
        assertEquals(Integer.valueOf(23), birthDateVO.getDay());

        birthDate.setValue("2018-01");
        birthDateVO = mapper.mapBirthDate(birthDate);
        assertEquals(Integer.valueOf(2018), birthDateVO.getYear());
        assertEquals(Integer.valueOf(1), birthDateVO.getMonth());
        assertNull(birthDateVO.getDay());

        birthDate.setValue("2018");
        birthDateVO = mapper.mapBirthDate(birthDate);
        assertEquals(Integer.valueOf(2018), birthDateVO.getYear());
        assertNull(birthDateVO.getMonth());
        assertNull(birthDateVO.getDay());
    }
}
